<?php

namespace Storage;

class Session
{
    public static function get($key, $default = null)
    {
        return array_key_exists($key, $_SESSION) ? $_SESSION[$key] : $default;
    }

    public static function set($key, $value)
    {
        return $_SESSION[$key] = $value;
    }

    public static function delete($key)
    {
        unset($_SESSION[$key]);
    }

    public static function clearAll()
    {
        $_SESSION = [];
    }

    public static function start(array $options, Cookie $cookie)
    {
        if ( session_status() === PHP_SESSION_ACTIVE ) {
            return;
        }

        $params = session_get_cookie_params();

        $params = [
            'expires'  => $options['lifetime'] ?? $params['lifetime'],
            'path'     => $options['path']     ?? $params['path'],
            'domain'   => $options['domain']   ?? $params['domain'],
            'secure'   => $options['secure']   ?? $params['secure'],
            'httponly' => $options['httponly'] ?? $params['httponly'],
            'samesite' => $options['samesite'] ?? $params['samesite'] ?? "",
        ];

        if ( version_compare(PHP_VERSION, '7.3.0') >= 0 ) {
            session_set_cookie_params($params);
        }
        else {
            session_set_cookie_params( ...array_values(array_slice($params, 0, 5)) );
        }

        session_name($options['name']);
        session_cache_limiter($options['cache_limiter'] ?? 'nocache');
        session_start();

        # Reset timeout after session started
        $cookie->set(session_name(), session_id(), time() + $params['expires'], $params['path'], $params['domain'], $params['secure'], $params['httponly'], $params['samesite']);
    }

    public static function regenerate()
    {
        if ( session_status() === PHP_SESSION_ACTIVE ) {
            session_regenerate_id(true);
        }
    }

    public static function destroy()
    {
        $this->clearAll();

        if ( ini_get("session.use_cookies") ) {
            $cookie->delete(session_name(), $params['path'] ?? "", $params['domain'] ?? "");
        }

        if ( session_status() === PHP_SESSION_ACTIVE ) {
            session_destroy();
        }
    }

    public function __set($key, $value)
    {
        return static::set($key, $value);
    }

    public function __get($key)
    {
        return static::get($key);
    }

    public function __isset($key)
    {
        return array_key_exists($key, $_SESSION);
    }

    public function __unset($key)
    {
        $this->delete($key);
    }
}
