<?php

namespace Storage;

use Psr\Http\Message\RequestInterface,
    Psr\Http\Message\ResponseInterface,
    Psr\Http\Server\MiddlewareInterface,
    Psr\Http\Message\ServerRequestInterface,
    Psr\Http\Server\RequestHandlerInterface;

class SessionMiddleware implements MiddlewareInterface
{
    protected array $options = [
        'name' => '',
        'path' => null,
        'domain' => null,
        'secure' => false,
        'httponly' => true,
        'lifetime' => 28800,  # A day of work ~ 7 hours
        'samesite' => "",
    ];

    protected Cookie $cookie;

    public function __construct(Cookie $cookie, array $options = [])
    {
        $this->cookie = $cookie;

        if ( empty($options['name']) ) {
            # unique directory-based named session
            $options['name'] = md5(__FILE__);
        }

        $this->options = array_merge($this->options, $options);
    }

    /**
     *
     * @param  RequestInterface        $request  PSR7 request object
     * @param  RequestHandlerInterface $response PSR7 handler object
     *
     * @return ResponseInterface PSR7 response object
     */
    public function process(ServerRequestInterface $request, RequestHandlerInterface $handler) : ResponseInterface
    {
        Session::start($this->options, $this->cookie);
        return $handler->handle($request);
    }
}
